'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Selector = require('./Selector');

var _Selector2 = _interopRequireDefault(_Selector);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var originPrecedence = ['user-important', 'author-important', 'author-normal', 'user-normal', 'userAgent'];
var stylesheetTypes = ['author', 'user', 'userAgent'];

var PropertyTest = function () {

	/**
  * Constructor
  *
  * @param  {Object} options plain object with options or Selector object
  */
	function PropertyTest() {
		var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck(this, PropertyTest);

		this.origin = 'inline';
		this.important = false;
		this.selector = null;

		// shortcut for Selector objects
		if (options instanceof _Selector2.default) {
			this.selector = options;
			this.origin = 'author';
		} else {
			Object.assign(this, options);
		}
	}

	/**
  * Get the specificity level of the selector.
  *
  * @return {object} specificity object with the number of selectors for each component type a,b,c,d
  */


	// default props


	_createClass(PropertyTest, [{
		key: 'getSpecificity',
		value: function getSpecificity() {
			if (stylesheetTypes.includes(this.origin) && this.selector instanceof _Selector2.default) {
				return this.selector.specificity;
			} else if (this.origin === 'inline' && this.selector === null) {
				return { a: 1, b: 0, c: 0, d: 0 };
			} else {
				throw new Error('Invalid CSS property test! Likely causes: invalid origin type or incompatible selector/origin combination.');
			}
		}

		/**
   * Get the precedence level of the selector's origin and !important status.
   *
   * @return {int} precedence level, lower being higher precedence
   */

	}, {
		key: 'getPrecedenceLevel',
		value: function getPrecedenceLevel() {
			// inline styles originate from the author
			var level,
			    precedenceType = this.origin === 'inline' ? 'author' : this.origin;

			// userAgent stylesheets cannot use "!important"
			if (this.origin !== 'userAgent') {
				precedenceType += '-' + (this.important === true ? 'important' : 'normal');
			}

			// derive the level from the origin precedence
			if ((level = originPrecedence.indexOf(precedenceType)) === -1) {
				throw new Error('Invalid origin type.');
			}

			return level;
		}
	}]);

	return PropertyTest;
}();

exports.default = PropertyTest;